/*****************************************************************************
 *****************************************************************************
 Copyright (c) 2000-2001, AEP Ltd. 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of AEP Ltd. nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

 *****************************************************************************
****************************************************************************/

/**********************************************************************
*                                                                     *
* AEP Ltd.                                                            *
*                                                                     *
* This software is supplied under the terms of the license included   *
* above.  All use of this software must be in accordance with the     *
* terms of that license.                                              *
*                                                                     *
**********************************************************************/

#include "load_utils.h"


/*******************************************************************************/
/*	OpenAccel( .. )

	Opens the destination device (target) in read/write mode.

	Input:
	acDevName	Name of the destination device to open

	Output:
	int			- Negative if an error occurred, successful otherwise
*/
signed int OpenAccel( char *acDevName ) {
	signed int iRetVal;
	
	iRetVal = open(acDevName, O_RDWR);
	
	if (iRetVal < 0) {
		fprintf(stderr, "\nError communicating with the accelerator %s: %s\n", acDevName, strerror(errno));
		if(errno==ENODEV) {
			/* a little hint for the user ... */
			fprintf(stderr, "Please check if driver is loaded\n");
		}
	}
	
	return iRetVal;
}



/*******************************************************************************/
/*	OpenImage( .. )

	Opens a source binary file (image) in read mode.

	Input:
	acImageName	- Name of the binary file to open

	Output:
	int			- Non-Zero if Successful, zero if an error occurred
*/
FILE * OpenImage( char *acImageName ) {
	FILE *fp;
	
	fp = fopen(acImageName, "rb");
	
	if (fp == NULL) {
		fprintf(stderr, "\nError opening image from file %s: %s\n", acImageName, strerror(errno));
	}
	
	return fp;
}




/*******************************************************************************/
/*	WriteInt( .. )

	Writes a 32-bit value to a destination address and converts its endiness
	if necessary.  NOTE: This routine should be used to write to a target only.

	Input:
	piDataToSend	Pointer to the destination address
	iValue			Data to write

	Output:
	None
*/
void WriteInt( AEP_U32 *piDataToSend, AEP_U32 iValue )
{
	*piDataToSend = iValue;
	
	if(bVerbose) printf("Data field (AEP_U32):\n Value = 0x%08x (%d)\n Size	 = %d (bytes)\n",
						*piDataToSend, *piDataToSend, sizeof(AEP_U32));
	
	if(bVerbose) printf("Converting endianness ...");
	BEndToLEnd( (AEP_U8 *)piDataToSend, 1);
	if(bVerbose) printf("done\n");
}




/*******************************************************************************/
/*	WriteMessage( .. )

	Sends a message contained in the char array portion of the AEP_BYTEBLOCK
	pCommandBlock to the destination device.

	Input:
	iDevFD			The file descriptor of the destination device (target)
	pCommandBlock	Pointer to a structure containing the length of the message
					to be sent as well as the data to be sent

	Output:
	signed int		The number of bytes written if successful, negative if error occurred
*/
signed int WriteMessage( int iDevFD, AEP_BYTEBLOCK *pCommandBlock )
{
	int iBytesWritten;
	
	if(bVerbose){	 
		printf("total message length: %d\n", pCommandBlock->Len);
		printf("writing message to accelerator...");
	}
	
	iBytesWritten = write(iDevFD, pCommandBlock->ptr, pCommandBlock->Len);
	
	if( iBytesWritten != pCommandBlock->Len ){
		return -1;
	}
	
	if(bVerbose) printf("done\n");
	fflush(stdout);
	
	return iBytesWritten;
}



/*******************************************************************************/
/*	GenerateMsgHeader( .. )

	Formats and writes a target StrongARM message header to an 8-bit array.
	This routine should only be used to write to the target as an endiness conversion
	maybe performed.

	Input:
	pacDest			The array to write the msg header information to
	iCommand		See message header documentation
	iStatus			See message header documentation
	iFlags			See message header documentation
	iDataLen		See message header documentation

	Output:
	None
*/
void GenerateMsgHeader( AEP_U8 *pacDest,
						AEP_U32 iCommand,
						AEP_U32 iStatus,
						AEP_U32 iFlags,
						AEP_U32 iDataLen )
{
	AEP_MSG_HDR_PTR psMessageHdr;
	
	psMessageHdr = (AEP_MSG_HDR_PTR)pacDest;
	
	psMessageHdr->app_id	 = (AEP_U64)getpid();  
	psMessageHdr->trans_hndl = (AEP_U64)getpid() << 32;
	psMessageHdr->command	 = iCommand;
	psMessageHdr->status	 = iStatus; 
	psMessageHdr->flags		 = iFlags;
	psMessageHdr->datalen	 = iDataLen;
	
	if(bVerbose){
		printf("************** Msg Header **************\n");
		printf("message header (AEP_MSG_HEADER): \n");
		printf(" psMessageHdr->app_id =		0x%016llx (%llu)\n",  psMessageHdr->app_id, psMessageHdr->app_id);
		printf(" psMessageHdr->trans_hndl = 0x%016llx (%llu)\n",  psMessageHdr->trans_hndl, psMessageHdr->trans_hndl);
		printf(" psMessageHdr->command =	0x%08x (%d)\n",	 psMessageHdr->command, psMessageHdr->command);
		printf(" psMessageHdr->status =		0x%08x (%d)\n",	 psMessageHdr->status,	psMessageHdr->status);
		printf(" psMessageHdr->flags =		0x%08x (%d)\n",	 psMessageHdr->flags,  psMessageHdr->flags);
		printf(" psMessageHdr->datalen =	0x%08x (%d)\n",	 psMessageHdr->datalen,	 psMessageHdr->datalen);
		printf("****************************************\n");
		fflush(stdout);
	}
	
	/* Convert endiness */
	BEndToLEnd( (AEP_U8 *)pacDest, HDR_SIZE/sizeof(AEP_U32) );
}

